package internal;

import internal.cic.ListBehaviour;
import internal.cic.RegistrationBehaviour;
import jade.content.ContentManager;
import jade.content.lang.sl.SLCodec;
import jade.content.onto.Ontology;
import jade.core.Agent;
import jade.util.leap.ArrayList;

import java.util.Iterator;

import misc.AgentHelper;
import ontology.CICAgentDescription;
import ontology.LogisticsOntology;

/**
 * 
 * Represents CIC service in Logistics System.
 * CIC service registers WhAs and answers product supplier queries from LA.
 *
 * Dependencies:
 * AgentHelper.getProductSuppliers
 * 
 */
public class CICAgent extends Agent {
	public SLCodec contentLanguage;
	public Ontology contentOntology;

	public ArrayList descriptionList; // list of registered WhAs (<CICAgentDescription>)

	// initialization
	protected void setup() {
		AgentHelper.debugMessage(this, "started");

		contentLanguage = new SLCodec();
		contentOntology = LogisticsOntology.getInstance();
		ContentManager manager = getContentManager();
		manager.registerLanguage(contentLanguage);
		manager.registerOntology(contentOntology);

		descriptionList = new ArrayList();

		// let cic respond to registration requests
		addBehaviour(new RegistrationBehaviour(this));
		// let cic respond to list queries
		addBehaviour(new ListBehaviour(this));
	}

	// perform registration/deregistration
	// register - true, deregister - false
	public void doRegistration(CICAgentDescription agentCICDescription, boolean register) {
		if (register) {
			this.descriptionList.add(agentCICDescription);
		} else {
			for (Iterator iter = descriptionList.iterator(); iter.hasNext();) {
				CICAgentDescription desc = (CICAgentDescription) iter.next();
				if (desc.getAgentAID().compareTo(agentCICDescription.getAgentAID()) == 0)
					iter.remove();
			}
		}
	}
}