package internal.cic;

import internal.CICAgent;
import jade.content.AgentAction;
import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.SimpleAchieveREResponder;
import jade.util.leap.ArrayList;
import jade.util.leap.List;

import java.util.Iterator;

import misc.AgentHelper;
import ontology.CICAgentDescription;
import ontology.CICQuery;
import ontology.CICResponse;
import ontology.Product;

// handle supplier listing requests
public class ListBehaviour extends SimpleAchieveREResponder {
	private final CICAgent theAgent;

	public ListBehaviour(CICAgent agent) {
		super(agent, MessageTemplate.and(
				createMessageTemplate(InteractionProtocol.FIPA_QUERY),
				MessageTemplate.and(MessageTemplate
						.MatchOntology(agent.contentOntology.getName()),
						MessageTemplate.MatchLanguage(agent.contentLanguage
								.getName()))));
		this.theAgent = agent;
	}

	protected ACLMessage prepareResponse(ACLMessage msg) {
		return null;
	}

	// prepare query response
	protected ACLMessage prepareResultNotification(ACLMessage request,
			ACLMessage response) {
		ACLMessage result = request.createReply();
		ContentElement content = null;
		ArrayList suppliers = null;
		AgentHelper.debugMessage(theAgent, "got list query from "
				+ request.getSender().getLocalName() + ", processing");

		// allow CIC to serve more queries while processing this one
		theAgent.addBehaviour(new ListBehaviour(theAgent));

		try {
			try {
				content = theAgent.getContentManager().extractContent(request);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent,
						"Error relating to message and " + "ontology: " + e);
				throw new NotUnderstoodException("(" + request.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error parsing the message format: " + e);
				throw new NotUnderstoodException("(" + request.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper.errorMessage(theAgent,
						"Message body was not an action");
				throw new NotUnderstoodException("(" + request.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent,
						"Message body was not present");
				throw new NotUnderstoodException("(" + request.toString() + ")");
			} else {
				if (action_content instanceof CICQuery) {
					// query action
					CICQuery information_request = (CICQuery) action_content;

					// get product identification data
					String theProduct = information_request.getTheProduct()
							.getProductName();
					// get suppliers of product
					suppliers = getProductSuppliers(theAgent, theProduct);
					result.setPerformative(ACLMessage.INFORM);

					// fill response
					CICResponse resp = new CICResponse();
					resp.setAgentList((List) suppliers);

					AgentAction respContent = resp;
					Action action = new Action(new AID(), respContent);

					try {
						// fill contents of response
						theAgent.getContentManager()
								.fillContent(result, action);
					} catch (CodecException e) {
						AgentHelper.errorMessage(theAgent,
								"Error relating to message and " + "ontology: "
										+ e);
						throw new NotUnderstoodException("("
								+ request.toString() + ")");
					} catch (OntologyException e) {
						AgentHelper.errorMessage(theAgent,
								"Error parsing the message format: " + e);
						throw new NotUnderstoodException("("
								+ request.toString() + ")");
					}
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + request.toString()
							+ ")");
				}
			}
		} catch (NotUnderstoodException e) {
			response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
			response.setContent(e.getMessage());
		}
		return result;
	}

	// get suppliers of given product
	private ArrayList getProductSuppliers(CICAgent theAgent, String product) {
		ArrayList suppliers = new ArrayList();

		Iterator iter1 = theAgent.descriptionList.iterator();
		Iterator iter2 = null;
		// parse registered suppliers list
		while (iter1.hasNext()) {
			CICAgentDescription desc = (CICAgentDescription) iter1.next();
			iter2 = desc.getAllOfferedProducts();
			// parse supplier's product list
			while (iter2.hasNext()) {
				Product p = (Product) iter2.next();
				if (p.getProductName().compareTo(product) == 0) {
					// product supplier found
					suppliers.add(desc.getAgentAID());
					break;
				}
			}
		}
		return suppliers;
	}
}