package internal.la;

import internal.LogisticsAgent;
import jade.content.AgentAction;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.OneShotBehaviour;
import jade.lang.acl.ACLMessage;
import misc.AgentHelper;
import ontology.InformResult;
import ontology.Result;
import ontology.ResultFailure;
import ontology.ResultSuccess;

// Last state of FSM. Handles result processing and notifies WA of the order
// results
public class FinalStateBehaviour extends OneShotBehaviour {
	private IssueOrderBehaviour theMaster;
	private final LogisticsAgent theAgent;

	public FinalStateBehaviour(IssueOrderBehaviour master) {
		theMaster = master;
		theAgent = theMaster.theAgent;
	}

	public void action() {
		boolean success = false;
		boolean beforeDeadline = false;

		// check if delivery confirmaton was received
		if (theMaster.theDelivery != null) {
			// YES - order success
			success = true;
			AgentHelper.debugMessage(theAgent,
					"ordering request successful");
		} else {
			// NO - order failed
			AgentHelper.debugMessage(theAgent,
					"ordering request failed");
		}

		// special behaviour while managing reliability if deadline is not
		// crossed
		if (theMaster.theDeadline >= 0) {
			beforeDeadline = true;
			AgentHelper.debugMessage(theAgent, "before deadline");
		}

		// manage reliability
		theAgent.manageReliability(theMaster.involvedSuppliers, success,
				beforeDeadline);

		// notify WA of order results
		ACLMessage orderRequestResult = prepareOrderRequestResult(theAgent, success,
				theMaster.orderRequestMessage);
		myAgent.send(orderRequestResult);
	}
	
	// prepare final result to WA
	private ACLMessage prepareOrderRequestResult(LogisticsAgent theAgent, boolean success,
			ACLMessage originalMessage) {
		// reply to the original WA message
		ACLMessage message = originalMessage.createReply();

		Result result = new Result();
		if (success) {
			// order successful
			result.setResultType(new ResultSuccess());
			message.setPerformative(ACLMessage.INFORM);
		} else {
			// order failed
			result.setResultType(new ResultFailure());
			message.setPerformative(ACLMessage.FAILURE);
		}
		InformResult inform = new InformResult();
		inform.setResult(result);

		AgentAction content = inform;
		Action action = new Action(theAgent.myWA, content);

		try {
			// fill contents of message
			theAgent.getContentManager().fillContent(message, action);
		} catch (CodecException e) {
			AgentHelper.errorMessage(theAgent, "Error relating to message and"
					+ "ontology");
			return null;
		} catch (OntologyException e) {
			AgentHelper.errorMessage(theAgent, "Error parsing the message format");
			return null;
		}
		return message;
	}
}
