package internal.la;

import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.Agent;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.lang.acl.ACLMessage;
import misc.AgentHelper;
import ontology.ConfirmationResponse;
import ontology.DeliveryDescription;
import ontology.OrderConfirmation;
import ontology.OrderDescription;
import ontology.OrderRequest;
import ontology.WADelivery;

/**
 * 
 * Contains helper methods for LA and its behaviours.
 *
 */
public class LAHelper {
	
	// extract confirmation from message passed from OA
	public static OrderConfirmation extractOrderConfirmation(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper
						.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof ConfirmationResponse) {

					ConfirmationResponse information_request = (ConfirmationResponse) action_content;

					// get confirmation details
					OrderConfirmation theConfirmation = information_request
							.getOrderConfirmation();
					return theConfirmation;
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return null;
	}
	
	// extract delivery details from WA notification
	public static DeliveryDescription extractDelivery(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper
						.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof WADelivery) {

					WADelivery information_request = (WADelivery) action_content;

					// get delivery details
					DeliveryDescription theDescription = information_request
							.getDeliveryDescription();
					return theDescription;
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return null;
	}
	
	// extract order description from WA message
	public static OrderDescription extractOrderDescription(Agent theAgent, ACLMessage msg) {
		ContentElement content = null;

		try {
			try {
				content = theAgent.getContentManager().extractContent(msg);
			} catch (OntologyException e) {
				AgentHelper.errorMessage(theAgent, "Error relating to message and "
						+ "ontology");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} catch (Codec.CodecException e) {
				AgentHelper.errorMessage(theAgent,
						"Error Parsing the message format");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			Concept action_content;

			if (content == null || !(content instanceof Action)) {
				AgentHelper
						.errorMessage(theAgent, "Message body was not an action");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			}

			action_content = ((Action) content).getAction();

			if (action_content == null) {
				AgentHelper.errorMessage(theAgent, "Message body was not present");
				throw new NotUnderstoodException("(" + msg.toString() + ")");
			} else {
				if (action_content instanceof OrderRequest) {

					OrderRequest information_request = (OrderRequest) action_content;

					// get order details
					OrderDescription theDescription = information_request
							.getOrderDescription();
					return theDescription;
				} else {
					AgentHelper.errorMessage(theAgent,
							"Message body was not a correct action");
					throw new NotUnderstoodException("(" + msg.toString() + ")");
				}
			}
		} catch (NotUnderstoodException e) {
			;
		}
		return null;
	}
}