package internal.la;

import internal.LogisticsAgent;
import jade.content.AgentAction;
import jade.content.lang.Codec;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.AID;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import jade.proto.SimpleAchieveREInitiator;

import java.util.Vector;

import ontology.OrderConfirmation;
import ontology.Reminder;

import misc.AgentHelper;

// handle sending reminders
public class ReminderBehaviour extends SimpleAchieveREInitiator {
	private int success = 1;

	private IssueOrderBehaviour theMaster;
	private final LogisticsAgent theAgent;

	private ACLMessage theMessage; // the message we sent and was accepted
	private AID oa; // OA we chose as target of message

	public ReminderBehaviour(IssueOrderBehaviour master, ACLMessage request) {
		super(master.theAgent, request);
		theMaster = master;
		theAgent = theMaster.theAgent;
	}

	// prepare reminder request
	protected ACLMessage prepareRequest(ACLMessage msg) {
		if (theMaster.currentOA < theAgent.oaPool.size()) {
			// we have some OAs available
			// prepare reminder message
			msg = prepareReminder(theMaster.theConfirmation);
			oa = (AID) theAgent.oaPool.get(theMaster.currentOA++);
			// addresss it
			msg.clearAllReceiver();
			msg.addReceiver(oa);
			// save for future reference
			theMessage = msg;
			return msg;
		} else {
			// no more OAs available
			// wait for them and retry from the first
			theMaster.currentOA = 0;
			block(theAgent.MONITORING_INTERVAL);
			// prepare reminder message
			msg = prepareReminder(theMaster.theConfirmation);
			oa = (AID) theMaster.theAgent.oaPool.get(theMaster.currentOA++);
			// addresss it
			msg.clearAllReceiver();
			msg.addReceiver(oa);
			// save for future reference
			theMessage = msg;
			return msg;
		}
	}

	public void reset(ACLMessage msg) {
		super.reset(msg);
	}

	// optional
	protected void handleAgree(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender()
				.getLocalName()
				+ " accepted the reminder request");
	}

	// OA busy?
	protected void handleRefuse(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender()
				.getLocalName()
				+ " is busy");
	}

	// handle timeout - OA busy?
	protected void handleAllResponses(Vector msgs) {
		AgentHelper.debugMessage(theAgent, oa.getLocalName()
				+ " did not respond, maybe it's busy");
	}

	// OA responded successfully
	protected void handleInform(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender()
				.getLocalName()
				+ " acknowledged the reminder");
		// extract new confirmation
		theMaster.theConfirmation = LAHelper.extractOrderConfirmation(myAgent, msg);
		// save original reminder request
		theMaster.reminderMessage = theMessage;
		// set new timeout
		theMaster.theTimeout = theMaster.theConfirmation
				.getOfferDescription().getDeliveryTimeSpecific();
		// save supplier
		AID chosenSupplier = theMaster.theConfirmation
				.getOfferDescription().getAgentAID();
		// save for reliability
		theMaster.involvedSuppliers.add(chosenSupplier);
		success = 0;
	}

	protected void handleNotUnderstood(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender()
				.getLocalName()
				+ " did not understand my request");
	}

	protected void handleOutOfSequence(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender()
				.getLocalName()
				+ " sent me unexpected message");
	}

	// WhA denied any responsibility
	protected void handleFailure(ACLMessage msg) {
		AgentHelper.debugMessage(theAgent, msg.getSender()
				.getLocalName()
				+ " failed to remind");
		success = 3;
	}

	// FSM result
	public int onEnd() {
		return success;
	}
	
	// prepare reminder template
	private ACLMessage prepareReminder(OrderConfirmation conf) {
		ACLMessage request = new ACLMessage(ACLMessage.REQUEST);
		request.setSender(theAgent.getAID());
		request.setLanguage(theAgent.contentLanguage.getName());
		request.setOntology(theAgent.contentOntology.getName());

		request.setProtocol(InteractionProtocol.FIPA_REQUEST);

		Reminder reminder = new Reminder();
		// set order confirmation we are referring to
		reminder.setOrderConfirmation(conf);

		AgentAction content = reminder;
		Action action = new Action(new AID(), content);

		try {
			AgentHelper.debugMessage(theAgent, "preparing reminder");
			// fill content
			theAgent.getContentManager().fillContent(request, action);
		} catch (OntologyException e) {
			AgentHelper.errorMessage(theAgent,
					"Error relating to message and ontology");
		} catch (Codec.CodecException e) {
			AgentHelper.errorMessage(theAgent, "Error Parsing the message format");
		}
		return request;
	}
}
