package internal.sda;

import internal.SDAgent;
import jade.content.AgentAction;
import jade.content.lang.Codec.CodecException;
import jade.content.onto.OntologyException;
import jade.content.onto.basic.Action;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.FIPANames.InteractionProtocol;
import jade.lang.acl.ACLMessage;
import misc.AgentHelper;
import ontology.PredictionDescription;
import ontology.SDAPrediction;

/**
 * 
 * Sends out a prediction.
 *
 */
public class PredictionBehaviour extends OneShotBehaviour {

	private PredictionDescription theDescription;
	private final SDAgent theAgent;

	public PredictionBehaviour(SDAgent theAgent, PredictionDescription description) {
		this.theAgent = theAgent;
		theDescription = description;
	}

	// sends the prediction to the WA
	public void action() {
		AgentHelper.debugMessage(theAgent, "sending prediction");

		ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
		msg.setSender(theAgent.getAID());
		msg.addReceiver(theAgent.myWA);
		msg.setLanguage(theAgent.contentLanguage.getName());
		msg.setOntology(theAgent.contentOntology.getName());
		msg.setProtocol(InteractionProtocol.FIPA_REQUEST);

		SDAPrediction prediction = new SDAPrediction();
		prediction.setPredictionDescription(theDescription);

		AgentAction content = prediction;
		Action action = new Action(theAgent.myWA, content);

		try {
			// fill contents of message
			theAgent.getContentManager().fillContent(msg, action);
		} catch (CodecException e) {
			AgentHelper.errorMessage(theAgent,
					"Error relating to message and " + "ontology: " + e);
			return;
		} catch (OntologyException e) {
			AgentHelper.errorMessage(theAgent,
					"Error Parsing the message format: " + e);
			return;
		}
		theAgent.send(msg);
	}
}